@@ -11,15 +11,15 @@
 
 using namespace llvm;
 
-// cl::opt<std::string> Inst1("inst1", cl::init("0"), cl::desc("first statement to mutate"));
 static cl::list<std::string, bool> InstList1("inst1", cl::desc("first statement to mutate"), cl::ZeroOrMore);
 cl::opt<std::string> Inst1ID("inst1ID", cl::init(""), cl::desc("The Unique ID of first statement to mutate"));
 
-// cl::opt<std::string> Inst2("inst2", cl::init("0"), cl::desc("second statement to mutate"));
 static cl::list<std::string, bool> InstList2("inst2", cl::desc("second statement to mutate"), cl::ZeroOrMore);
 cl::opt<std::string> Inst2ID("inst2ID", cl::init(""), cl::desc("The Unique ID of first statement to mutate"));
 
 cl::opt<bool> use_result("use_result", cl::init(1), cl::desc("Whether to perform the use_result after changing the instruction"));
+cl::opt<bool> deterministic("deterministic", cl::init(0), cl::desc("Whether the random process needs to be deterministic"));
+cl::opt<int> rnd_seed("seed", cl::init(1), cl::desc("seed for the random sqeuence if deterministic is set. "));
 
 namespace {
   struct Ids : public ModulePass {
@@ -196,6 +196,9 @@ namespace {
     Cut() : ModulePass(ID) {}
 
     bool runOnModule(Module &M){
+      if (deterministic)
+        gen.seed(rnd_seed);
+
       std::string Inst1 = InstList1[0];
       InstList1.erase(InstList1.begin());
       InstList2.erase(InstList2.begin());
@@ -209,7 +212,7 @@ namespace {
       // This cannot be omitted, otherwise the llvm-dis will have segfault
       if(!I->use_empty()){
         std::pair<Value*, StringRef> Val =
-          randValueBeforeI(I->getFunction(), I, cast<Value>(I));
+          randValueBeforeI(I->getFunction(), I, cast<Value>(I), deterministic);
         if(Val.first != NULL)
           replaceAllUsesWithReport(I, Val);
       }
@@ -226,6 +229,9 @@ namespace {
     Insert() : ModulePass(ID) {}
 
     bool runOnModule(Module &M){
+      if (deterministic)
+        gen.seed(rnd_seed);
+
       Instruction *temp;
       std::string Inst1 = InstList1[0];
       std::string Inst2 = InstList2[0];
@@ -247,13 +253,13 @@ namespace {
 
       temp->insertBefore(&*DI); // insert temp before DI
       updateMetadata(temp, "i");
-      replaceUnfulfillOperands(temp); // wire incoming edges of CFG into temp
+      replaceUnfulfillOperands(temp, deterministic); // wire incoming edges of CFG into temp
       // check if I generates a result which is used, if not then
       // it is probably run for side effects and we don't need to
       // wire the result of the copy of I to something later on
       bool result_ignorable_p = SI->use_empty();
       if(!result_ignorable_p && use_result)
-        useResult(temp); // wire outgoing results of temp into CFG
+        useResult(temp, deterministic); // wire outgoing results of temp into CFG
 
       errs()<<"inserted " << Inst1ID << "," << Inst2ID << "\n";
       return RET_SUCCESS;
@@ -267,6 +273,9 @@ namespace {
     Replace() : ModulePass(ID) {}
 
     bool runOnModule(Module &M){
+      if (deterministic)
+        gen.seed(rnd_seed);
+
       Instruction *temp;
       std::string Inst1 = InstList1[0];
       std::string Inst2 = InstList2[0];
@@ -293,7 +302,7 @@ namespace {
       insertNOP(&*DI);
       ReplaceInstWithInst(DI, temp);
       updateMetadata(temp, "r");
-      replaceUnfulfillOperands(temp);
+      replaceUnfulfillOperands(temp, deterministic);
 
       errs()<<"replaced "<< Inst1ID << "," << Inst2ID << "\n";
       return RET_SUCCESS; }
@@ -306,6 +315,9 @@ namespace {
     OPRepl() : ModulePass(ID) {}
 
     bool runOnModule(Module &M){
+      if (deterministic)
+        gen.seed(rnd_seed);
+
       int err = 0;
       std::string Inst1 = InstList1[0];
       std::string Inst2 = InstList2[0];
@@ -323,13 +335,15 @@ namespace {
           result = randOperandAfterI(
             *cast<Argument>(sval)->getParent(),
             NULL, // search from the start of function
-            sval->getType()
+            sval->getType(),
+            deterministic
           );
         else
           result = randOperandAfterI(
             *cast<Instruction>(sval)->getFunction(),
             cast<Instruction>(sval),
-            sval->getType()
+            sval->getType(),
+            deterministic
           );
         if (result.first == NULL) {
           errs() << "oprepl failed("<< Inst1 << "," << Inst2 << "). cannot find an OP that has the same type as " << Inst2 << "\n";
@@ -356,7 +370,7 @@ namespace {
         }
         Inst1ID = Inst1ID + ".OP" + std::to_string(OPidx);
         Value *Dop = DI->getOperand(OPidx);
-        std::pair<Value*, StringRef> sval = randValueBeforeI(DI->getFunction(), DI, Dop);
+        std::pair<Value*, StringRef> sval = randValueBeforeI(DI->getFunction(), DI, Dop, deterministic);
         DI->setOperand(OPidx, sval.first);
         Inst2ID = sval.second.str();
 
@@ -364,7 +378,7 @@ namespace {
         return RET_SUCCESS;
       }
       else if (Inst1 == "Rand" && Inst2 == "Rand") {
-        std::pair<Value*, StringRef> val = randValue(M);
+        std::pair<Value*, StringRef> val = randValue(M, deterministic);
         Value *sval = val.first;
         Inst2ID = val.second.str();
         std::pair<Instruction*, unsigned> result;
@@ -372,13 +386,15 @@ namespace {
           result = randOperandAfterI(
             *cast<Argument>(sval)->getParent(),
             NULL, // search from the start of function
-            sval->getType()
+            sval->getType(),
+            deterministic
           );
         else
           result = randOperandAfterI(
             *cast<Instruction>(sval)->getFunction(),
             cast<Instruction>(sval),
-            sval->getType()
+            sval->getType(),
+            deterministic
           );
         if (result.first == NULL) {
           errs() << "oprepl failed("<< Inst1 << "," << Inst2 << "). cannot find an OP that has the same type as " << Inst2ID << "\n";
@@ -427,6 +443,9 @@ namespace {
     Move() : ModulePass(ID) {}
 
     bool runOnModule(Module &M){
+      if (deterministic)
+        gen.seed(rnd_seed);
+
       std::string Inst1 = InstList1[0];
       std::string Inst2 = InstList2[0];
       InstList1.erase(InstList1.begin());
@@ -450,14 +469,14 @@ namespace {
       updateMetadata(temp, "m");
       insertNOP(SI);
 
-      replaceUnfulfillOperands(temp);
+      replaceUnfulfillOperands(temp, deterministic);
       bool result_ignorable_p = SI->use_empty();
       if(!result_ignorable_p && use_result)
-        useResult(temp);
+        useResult(temp, deterministic);
       // Delete the source instruction if it is there
       if(!SI->use_empty()){
         std::pair<Value*, StringRef> Val =
-          randValueBeforeI(SI->getFunction(), SI, cast<Value>(SI));
+          randValueBeforeI(SI->getFunction(), SI, cast<Value>(SI), deterministic);
         if(Val.first != NULL)
           replaceAllUsesWithReport(SI, Val);
       }
@@ -476,6 +495,9 @@ namespace {
     Swap() : ModulePass(ID) {}
 
     bool runOnModule(Module &M){
+      if (deterministic)
+        gen.seed(rnd_seed);
+
       std::string Inst1 = InstList1[0];
       std::string Inst2 = InstList2[0];
       InstList1.erase(InstList1.begin());
@@ -510,14 +532,14 @@ namespace {
       bool result_ignorable_p2 = I2->use_empty();
       ReplaceInstWithInst(I2, temp1);
       updateMetadata(temp1, "s");
-      replaceUnfulfillOperands(temp1);
+      replaceUnfulfillOperands(temp1, deterministic);
       if(!result_ignorable_p1 && use_result)
-        useResult(temp1);
+        useResult(temp1, deterministic);
       ReplaceInstWithInst(I1, temp2);
       updateMetadata(temp2, "s");
-      replaceUnfulfillOperands(temp2);
+      replaceUnfulfillOperands(temp2, deterministic);
       if(!result_ignorable_p2 && use_result)
-        useResult(temp2);
+        useResult(temp2, deterministic);
 
       errs()<<"swapped " << Inst1ID << "," << Inst2ID << "\n";
 
@@ -530,13 +552,16 @@ namespace {
     static char ID; // pass identification
     Cache() : ModulePass(ID) {}
     bool runOnModule(Module &M){
+      if (deterministic)
+        gen.seed(rnd_seed);
+
       std::string Inst1 = InstList1[0];
       std::string Inst2 = InstList2[0];
       InstList1.erase(InstList1.begin());
       InstList2.erase(InstList2.begin());
       Instruction *I;
       if (Inst1 == "Rand") {
-        std::pair<Instruction*, StringRef> result = randTexCachableI(M);
+        std::pair<Instruction*, StringRef> result = randTexCachableI(M, deterministic);
         I = result.first;
         Inst1ID = result.second.str();
       }
