@@ -27,6 +27,21 @@ std::random_device rd;
 std::mt19937 gen(rd());
 const std::string ldgPre = "llvm.nvvm.ldg.global";
 
+// TODO: find a better but determinisic distribution function accross compilers
+class modulo_distribution {
+public:
+    modulo_distribution(int lowerBound, int upperBound) {
+        lb = lowerBound;
+        ub = upperBound;
+    }
+
+    unsigned int operator()(std::mt19937 &urng) {
+        return urng()%(ub+1-lb)+lb;
+    }
+private:
+    int lb, ub;
+};
+
 StringRef getUID(const Instruction *I) {
     if (!I)
         assert(0);
@@ -66,7 +81,7 @@ Value* getConstantValue(Type* T)
     case Type::StructTyID:    case Type::ArrayTyID:
         return Constant::getNullValue(T);
     default:
-        assert(0);
+        assert(0); exit(1);
     }
 }
 
@@ -109,7 +124,7 @@ void CollectValueBeforeI(Function *F, Instruction* boundary, Value* refOP,
     }
 }
 
-std::pair<Value*, StringRef> randValue(Module &M)
+std::pair<Value*, StringRef> randValue(Module &M, bool is_deterministic)
 {
     std::vector<std::pair<Value*, StringRef>> resultVec;
     for (Function &F : M) {
@@ -118,19 +133,36 @@ std::pair<Value*, StringRef> randValue(Module &M)
         CollectValueBeforeI(&F, NULL, NULL, resultVec);
     }
 
-    std::uniform_int_distribution<> randIdx(0, resultVec.size()-1);
-    return resultVec[randIdx(gen)];
+    unsigned int randIdx;
+    if (is_deterministic) {
+        modulo_distribution dist_func(0, resultVec.size()-1);
+        randIdx = dist_func(gen);
+    }
+    else {
+        std::uniform_int_distribution<> dist_func(0, resultVec.size()-1);
+        randIdx = dist_func(gen);
+    }
+
+    return resultVec[randIdx];
 }
 
-std::pair<Value*, StringRef> randValueBeforeI(Function *F, Instruction* boundary, Value* refOP)
+std::pair<Value*, StringRef> randValueBeforeI(Function *F, Instruction* boundary, Value* refOP, bool is_deterministic)
 {
     std::vector<std::pair<Value*, StringRef>> resultVec;
     CollectValueBeforeI(F, boundary, refOP, resultVec);
     // has constant to participate in drawing
     resultVec.push_back(std::make_pair(getConstantValue(refOP->getType()), StringRef("C1")));
 
-    std::uniform_int_distribution<> randIdx(0, resultVec.size()-1);
-    return resultVec[randIdx(gen)];
+    unsigned int randIdx;
+    if (is_deterministic) {
+        modulo_distribution dist_func(0, resultVec.size()-1);
+        randIdx = dist_func(gen);
+    }
+    else {
+        std::uniform_int_distribution<> dist_func(0, resultVec.size()-1);
+        randIdx = dist_func(gen);
+    }
+    return resultVec[randIdx];
 }
 
 void CollectOperandAfterI(Function &F, Instruction* boundary, Type* T,
@@ -194,7 +226,7 @@ void CollectOperandAfterI(Function &F, Instruction* boundary, Type* T,
 }
 
 //Note: Type T is implicitely extracted from the boundary instruction
-std::pair<Instruction*, unsigned> randOperandAfterI(Function &F, Instruction* boundary, Type* T) 
+std::pair<Instruction*, unsigned> randOperandAfterI(Function &F, Instruction* boundary, Type* T, bool is_deterministic) 
 {
     std::vector<std::pair<Instruction*, unsigned>> OPvec;
     CollectOperandAfterI(F, boundary, T, OPvec);
@@ -203,11 +235,19 @@ std::pair<Instruction*, unsigned> randOperandAfterI(Function &F, Instruction* bo
     if (OPvec.empty())
         return std::make_pair(dummy, 0);
 
-    std::uniform_int_distribution<> randIdx(0, OPvec.size()-1);
-    return OPvec[randIdx(gen)];
+    unsigned int randIdx;
+    if (is_deterministic) {
+        modulo_distribution dist_func(0, OPvec.size()-1);
+        randIdx = dist_func(gen);
+    }
+    else {
+        std::uniform_int_distribution<> dist_func(0, OPvec.size()-1);
+        randIdx = dist_func(gen);
+    }
+    return OPvec[randIdx];
 }
 
-std::pair<Instruction*, StringRef> randTexCachableI(Module &M)
+std::pair<Instruction*, StringRef> randTexCachableI(Module &M, bool is_deterministic)
 {
     std::vector<std::pair<Instruction*, StringRef>> resultVec;
     for (Function &F : M)
@@ -245,14 +285,22 @@ std::pair<Instruction*, StringRef> randTexCachableI(Module &M)
         }
     }
 
-    std::uniform_int_distribution<> randIdx(0, resultVec.size()-1);
-    return resultVec[randIdx(gen)];
+    unsigned int randIdx;
+    if (is_deterministic) {
+        modulo_distribution dist_func(0, resultVec.size()-1);
+        randIdx = dist_func(gen);
+    }
+    else {
+        std::uniform_int_distribution<> dist_func(0, resultVec.size()-1);
+        randIdx = dist_func(gen);
+    }
+    return resultVec[randIdx];
 }
 
 // Force to use the result of instruction tI somewhere later.
-void useResult(Instruction *tI){
+void useResult(Instruction *tI, bool is_deterministic){
     std::pair<Instruction*, unsigned> result;
-    result = randOperandAfterI(*(tI->getFunction()), tI, tI->getType());
+    result = randOperandAfterI(*(tI->getFunction()), tI, tI->getType(), is_deterministic);
 
     if (result.first == NULL) {
         errs()<<"could find no use for result\n";
@@ -295,7 +343,7 @@ bool dominates(Value* defV, Instruction *I)
 
 // Replace the operands of Instruction I with in-scope values of the
 // same type.  If the operands are already in scope, then retain them.
-void replaceUnfulfillOperands(Instruction *I){
+void replaceUnfulfillOperands(Instruction *I, bool is_deterministic){
     if(isa<BranchInst>(I)) return;
 
     // loop through operands,
@@ -305,7 +353,7 @@ void replaceUnfulfillOperands(Instruction *I){
         if (dominates(op.get(), I))
             continue;
 
-        std::pair<Value*, StringRef> ret = randValueBeforeI(I->getFunction(), I, op.get());
+        std::pair<Value*, StringRef> ret = randValueBeforeI(I->getFunction(), I, op.get(), is_deterministic);
         Value *val = ret.first;
 
         if(val != 0) {
@@ -372,7 +420,7 @@ bool iterInstComb(std::fstream &listf, Instruction *SIclone, Instruction *SI, un
         }
     }
     else {
-        assert(0);
+        assert(0); exit(1);
     }
 }
 
@@ -732,5 +780,5 @@ unsigned int getAlignFromType(Module &M, Type *T)
     else if (T == Type::getDoublePtrTy(M.getContext()))
         return 8;
     else
-        assert(0);
+        assert(0); exit(1);
 }
\ No newline at end of file
