@@ -86,6 +86,10 @@ if __name__ == '__main__':
               chain when performing mutation operations. This argument is mainly for reproducing\
               program variant from a sequence of mutations')
     llvm_mutate_parser.add_argument('--interrupt_at_fail', action='store_true')
+    llvm_mutate_parser.add_argument('-d', '--deterministic', action='store_true', 
+        help='make all the mutations and repairs deterministic')
+    llvm_mutate_parser.add_argument('--seed', type=int, 
+        help='set the random seed (only when deterministic is set to true)')
     llvm_mutate_parser.add_argument('-V', '--version', action='version', version='%(prog)s under gevo-' + __version__)
 
     # grouping mutation commands
@@ -173,9 +177,14 @@ if __name__ == '__main__':
         with args.input_file.open() as f:
             input_str = f.buffer.read()
 
-        opt_args = [opt_path, '-load', LLVM_MUTATE_LIBRARY_PATH,
-                    '-use_result=0'] if args.not_use_result else\
-                   [opt_path,'-load', LLVM_MUTATE_LIBRARY_PATH, ]
+        opt_args = [opt_path, '-load', LLVM_MUTATE_LIBRARY_PATH]
+        if args.not_use_result:
+            opt_args.extend(['-use_result=0'])
+        if args.deterministic:
+            opt_args.extend(['-deterministic=true'])
+            if args.seed:
+                opt_args.extend([f'-seed={str(args.seed)}'])
+
         for mop in args.mutation_commands:
             insts = mop[1].split(',')
             inst_args = [ mop[0], f'-inst1={insts[0]}', '-inst2=' ] if len(insts) == 1 else\
